from collections.abc import Callable, Iterable, Sequence
from datetime import datetime, timedelta
from typing import Any

from nonebot.adapters import MessageTemplate

from nonebot.dependencies import Dependent
from nonebot.internal.matcher import (
    Matcher,
    current_bot,
    current_event,
    current_matcher,
)
from nonebot.internal.params import (
    ArgParam,
    BotParam,
    DefaultParam,
    DependParam,
    Depends,
    EventParam,
    MatcherParam,
    StateParam,
)
from nonebot.permission import Permission
from nonebot.plugin.on import get_matcher_module, get_matcher_plugin, store_matcher
from nonebot.rule import Rule
from nonebot.typing import T_Handler, T_PermissionChecker, T_RuleChecker, T_State

from .params import OptionParam
from .storage import (
    _application_command_storage,
)
from ..api import (
    UNSET,
    AnyCommandOption,
    ApplicationCommandCreate,
    ApplicationCommandOptionType,
    ApplicationCommandType,
    InteractionCallbackType,
    InteractionResponse,
    MessageFlag,
    MessageGet,
    Missing,
    Snowflake,
    SnowflakeType,
)
from ..api.types import is_unset
from ..bot import Bot
from ..event import ApplicationCommandInteractionEvent
from ..message import Message, MessageSegment, parse_message

type_str_mapping = {
    ApplicationCommandOptionType.USER: "users",
    ApplicationCommandOptionType.CHANNEL: "channels",
    ApplicationCommandOptionType.ROLE: "roles",
    ApplicationCommandOptionType.ATTACHMENT: "attachments",
}


class ApplicationCommandConfig(ApplicationCommandCreate):
    guild_ids: list[Snowflake] | None = None


class ApplicationCommandMatcher(Matcher):
    application_command: ApplicationCommandConfig

    @classmethod
    async def send_deferred_response(cls) -> None:
        event = current_event.get()
        bot = current_bot.get()
        if not isinstance(event, ApplicationCommandInteractionEvent) or not isinstance(
            bot, Bot
        ):
            msg = "Invalid event or bot"
            raise ValueError(msg)  # noqa: TRY004
        await bot.create_interaction_response(
            interaction_id=event.id,
            interaction_token=event.token,
            response=InteractionResponse(
                type=InteractionCallbackType.DEFERRED_CHANNEL_MESSAGE_WITH_SOURCE
            ),
        )

    @classmethod
    async def send_response(
        cls, message: str | Message | MessageSegment | MessageTemplate
    ) -> None:
        return await cls.send(message)

    @classmethod
    async def get_response(cls) -> MessageGet:
        event = current_event.get()
        bot = current_bot.get()
        if not isinstance(event, ApplicationCommandInteractionEvent) or not isinstance(
            bot, Bot
        ):
            msg = "Invalid event or bot"
            raise ValueError(msg)  # noqa: TRY004
        return await bot.get_origin_interaction_response(
            application_id=event.application_id,
            interaction_token=event.token,
        )

    @classmethod
    async def edit_response(
        cls,
        message: str | Message | MessageSegment | MessageTemplate,
    ) -> None:
        event = current_event.get()
        bot = current_bot.get()
        state = current_matcher.get().state
        if not isinstance(event, ApplicationCommandInteractionEvent) or not isinstance(
            bot, Bot
        ):
            msg = "Invalid event or bot"
            raise ValueError(msg)  # noqa: TRY004
        if isinstance(message, MessageTemplate):
            _message = message.format(**state)
        else:
            _message = message
        message_data = parse_message(_message)
        await bot.edit_origin_interaction_response(
            application_id=event.application_id,
            interaction_token=event.token,
            **message_data,
        )

    @classmethod
    async def delete_response(cls) -> None:
        event = current_event.get()
        bot = current_bot.get()
        if not isinstance(event, ApplicationCommandInteractionEvent) or not isinstance(
            bot, Bot
        ):
            msg = "Invalid event or bot"
            raise ValueError(msg)  # noqa: TRY004
        await bot.delete_origin_interaction_response(
            application_id=event.application_id,
            interaction_token=event.token,
        )

    @classmethod
    async def send_followup_msg(
        cls,
        message: str | Message | MessageSegment | MessageTemplate,
        flags: MessageFlag | None = None,
    ) -> MessageGet:
        event = current_event.get()
        bot = current_bot.get()
        state = current_matcher.get().state
        if not isinstance(event, ApplicationCommandInteractionEvent) or not isinstance(
            bot, Bot
        ):
            msg = "Invalid event or bot"
            raise ValueError(msg)  # noqa: TRY004
        if isinstance(message, MessageTemplate):
            _message = message.format(**state)
        else:
            _message = message
        message_data = parse_message(_message)
        if flags:
            message_data["flags"] = int(flags)
        return await bot.create_followup_message(
            application_id=event.application_id,
            interaction_token=event.token,
            **message_data,
        )

    @classmethod
    async def get_followup_msg(cls, message_id: SnowflakeType) -> MessageGet:
        event = current_event.get()
        bot = current_bot.get()
        if not isinstance(event, ApplicationCommandInteractionEvent) or not isinstance(
            bot, Bot
        ):
            msg = "Invalid event or bot"
            raise ValueError(msg)  # noqa: TRY004
        return await bot.get_followup_message(
            application_id=event.application_id,
            interaction_token=event.token,
            message_id=message_id,
        )

    @classmethod
    async def edit_followup_msg(
        cls,
        message_id: SnowflakeType,
        message: str | Message | MessageSegment | MessageTemplate,
    ) -> MessageGet:
        event = current_event.get()
        bot = current_bot.get()
        state = current_matcher.get().state
        if not isinstance(event, ApplicationCommandInteractionEvent) or not isinstance(
            bot, Bot
        ):
            msg = "Invalid event or bot"
            raise ValueError(msg)  # noqa: TRY004
        if isinstance(message, MessageTemplate):
            _message = message.format(**state)
        else:
            _message = message
        message_data = parse_message(_message)
        return await bot.edit_followup_message(
            application_id=event.application_id,
            interaction_token=event.token,
            message_id=message_id,
            **message_data,
        )

    @classmethod
    async def delete_followup_msg(cls, message_id: SnowflakeType) -> None:
        event = current_event.get()
        bot = current_bot.get()
        if not isinstance(event, ApplicationCommandInteractionEvent) or not isinstance(
            bot, Bot
        ):
            msg = "Invalid event or bot"
            raise ValueError(msg)  # noqa: TRY004
        await bot.delete_followup_message(
            application_id=event.application_id,
            interaction_token=event.token,
            message_id=message_id,
        )


class SlashCommandMatcher(ApplicationCommandMatcher):
    HANDLER_PARAM_TYPES = (
        DependParam,
        BotParam,
        EventParam,
        StateParam,
        ArgParam,
        MatcherParam,
        DefaultParam,
        OptionParam,
    )

    @classmethod
    def handle_sub_command(
        cls, *commands: str, parameterless: Iterable[Any] | None = None
    ) -> Callable[..., T_Handler]:
        def _sub_command_rule(
            event: ApplicationCommandInteractionEvent,
            matcher: Matcher,
            state: T_State,  # noqa: ARG001 # TODO)): 验证state作用
        ) -> None:
            options = event.data.options
            if commands and is_unset(options):
                matcher.skip()
            for command in commands:
                if is_unset(options):
                    matcher.skip()
                option = options[0]
                if option.name != command or option.type not in (
                    ApplicationCommandOptionType.SUB_COMMAND_GROUP,
                    ApplicationCommandOptionType.SUB_COMMAND,
                ):
                    matcher.skip()
                options = option.options

        parameterless = [Depends(_sub_command_rule), *(parameterless or [])]

        def _decorator(func: T_Handler) -> T_Handler:
            cls.append_handler(func, parameterless=parameterless)
            return func

        return _decorator


class UserMessageCommandMatcher(ApplicationCommandMatcher):
    pass


def on_slash_command(  # noqa: PLR0913
    name: str,
    description: str,
    options: Missing[Sequence[AnyCommandOption]] = UNSET,
    internal_id: str | None = None,
    rule: Rule | T_RuleChecker | None = None,
    permission: Permission | T_PermissionChecker | None = None,
    *,
    name_localizations: dict[str, str] | None = None,
    description_localizations: dict[str, str] | None = None,
    default_member_permissions: str | None = None,
    dm_permission: bool | None = None,
    default_permission: Missing[bool] = UNSET,
    nsfw: Missing[bool] = UNSET,
    handlers: list[T_Handler | Dependent] | None = None,
    temp: bool = False,
    expire_time: datetime | timedelta | None = None,
    priority: int = 1,
    block: bool = True,
    state: T_State | None = None,
    _depth: int = 0,
) -> type[SlashCommandMatcher]:
    config = ApplicationCommandConfig(
        type=ApplicationCommandType.CHAT_INPUT,
        name=name,
        name_localizations=name_localizations,
        description=description,
        description_localizations=description_localizations,
        options=UNSET if is_unset(options) else list(options),
        default_member_permissions=default_member_permissions,
        dm_permission=dm_permission,
        default_permission=default_permission,
        nsfw=nsfw,
    )
    _application_command_storage[internal_id or name] = config
    matcher: type[SlashCommandMatcher] = SlashCommandMatcher.new(
        "notice",
        Rule() & rule,
        Permission() | permission,
        handlers=handlers,
        temp=temp,
        expire_time=expire_time,
        priority=priority,
        block=block,
        default_state=state,
        plugin=get_matcher_plugin(_depth + 1),
        module=get_matcher_module(_depth + 1),
    )

    def _application_command_rule(event: ApplicationCommandInteractionEvent) -> bool:
        if event.data.name != config.name or event.data.type != config.type:
            return False
        if not event.data.guild_id and config.guild_ids is None:
            return True
        return bool(
            event.data.guild_id
            and config.guild_ids
            and event.data.guild_id in config.guild_ids
        )

    matcher.rule = matcher.rule & Rule(_application_command_rule)

    store_matcher(matcher)
    matcher.application_command = config
    return matcher


def on_user_command(  # noqa: PLR0913
    name: str,
    internal_id: str | None = None,
    rule: Rule | T_RuleChecker | None = None,
    permission: Permission | T_PermissionChecker | None = None,
    *,
    name_localizations: dict[str, str] | None = None,
    default_member_permissions: str | None = None,
    dm_permission: bool | None = None,
    default_permission: Missing[bool] = UNSET,
    nsfw: Missing[bool] = UNSET,
    handlers: list[T_Handler | Dependent] | None = None,
    temp: bool = False,
    expire_time: datetime | timedelta | None = None,
    priority: int = 1,
    block: bool = True,
    state: T_State | None = None,
    _depth: int = 0,
) -> type[UserMessageCommandMatcher]:
    config = ApplicationCommandConfig(
        type=ApplicationCommandType.USER,
        name=name,
        name_localizations=name_localizations,
        default_member_permissions=default_member_permissions,
        dm_permission=dm_permission,
        default_permission=default_permission,
        nsfw=nsfw,
    )
    _application_command_storage[internal_id or name] = config
    matcher: type[UserMessageCommandMatcher] = UserMessageCommandMatcher.new(
        "notice",
        Rule() & rule,
        Permission() | permission,
        handlers=handlers,
        temp=temp,
        expire_time=expire_time,
        priority=priority,
        block=block,
        default_state=state,
        plugin=get_matcher_plugin(_depth + 1),
        module=get_matcher_module(_depth + 1),
    )

    def _application_command_rule(event: ApplicationCommandInteractionEvent) -> bool:
        if event.data.name != config.name or event.data.type != config.type:
            return False
        if not event.data.guild_id and config.guild_ids is None:
            return True
        return bool(
            event.data.guild_id
            and config.guild_ids
            and event.data.guild_id in config.guild_ids
        )

    matcher.rule = matcher.rule & Rule(_application_command_rule)

    store_matcher(matcher)
    matcher.application_command = config
    return matcher


def on_message_command(  # noqa: PLR0913
    name: str,
    internal_id: str | None = None,
    rule: Rule | T_RuleChecker | None = None,
    permission: Permission | T_PermissionChecker | None = None,
    *,
    name_localizations: dict[str, str] | None = None,
    default_member_permissions: str | None = None,
    dm_permission: bool | None = None,
    default_permission: Missing[bool] = UNSET,
    nsfw: Missing[bool] = UNSET,
    handlers: list[T_Handler | Dependent] | None = None,
    temp: bool = False,
    expire_time: datetime | timedelta | None = None,
    priority: int = 1,
    block: bool = True,
    state: T_State | None = None,
    _depth: int = 0,
) -> type[UserMessageCommandMatcher]:
    config = ApplicationCommandConfig(
        type=ApplicationCommandType.MESSAGE,
        name=name,
        name_localizations=name_localizations,
        default_member_permissions=default_member_permissions,
        dm_permission=dm_permission,
        default_permission=default_permission,
        nsfw=nsfw,
    )
    _application_command_storage[internal_id or name] = config
    matcher: type[UserMessageCommandMatcher] = UserMessageCommandMatcher.new(
        "notice",
        Rule() & rule,
        Permission() | permission,
        handlers=handlers,
        temp=temp,
        expire_time=expire_time,
        priority=priority,
        block=block,
        default_state=state,
        plugin=get_matcher_plugin(_depth + 1),
        module=get_matcher_module(_depth + 1),
    )

    def _application_command_rule(event: ApplicationCommandInteractionEvent) -> bool:
        if event.data.name != config.name or event.data.type != config.type:
            return False
        if not event.data.guild_id and config.guild_ids is None:
            return True
        return bool(
            event.data.guild_id
            and config.guild_ids
            and event.data.guild_id in config.guild_ids
        )

    matcher.rule = matcher.rule & Rule(_application_command_rule)

    store_matcher(matcher)
    matcher.application_command = config
    return matcher



from typing import Any
import zlib

from nonebot.compat import model_dump as model_dump_
from nonebot.utils import logger_wrapper
from pydantic import BaseModel

from .api.types import UNSET

log = logger_wrapper("Discord")


def omit_unset(data: Any) -> Any:  # noqa: ANN401
    """Recursively omit fields whose value is exactly ``UNSET``."""

    if isinstance(data, dict):
        return data.__class__(
            (k, omit_unset(v)) for k, v in data.items() if v is not UNSET
        )
    if isinstance(data, (list, tuple, set)):
        return data.__class__(omit_unset(i) for i in data if i is not UNSET)
    return data


def model_dump(  # noqa: PLR0913
    model: BaseModel,
    include: set[str] | None = None,
    exclude: set[str] | None = None,
    *,
    by_alias: bool = False,
    exclude_unset: bool = False,
    exclude_defaults: bool = False,
    exclude_none: bool = False,
    omit_unset_values: bool = False,
) -> dict[str, Any]:
    data = model_dump_(
        model,
        include=include,
        exclude=exclude,
        by_alias=by_alias,
        exclude_unset=exclude_unset,
        exclude_defaults=exclude_defaults,
        exclude_none=exclude_none,
    )
    if omit_unset_values:
        return omit_unset(data)
    return data


def escape(s: str) -> str:
    return s.replace("&", "&amp;").replace("<", "&lt;").replace(">", "&gt;")


def unescape(s: str) -> str:
    return s.replace("&lt;", "<").replace("&gt;", ">").replace("&amp;", "&")


def decompress_data(data: str | bytes, *, compress: bool) -> str | bytes:
    if not compress:
        return data
    if isinstance(data, str):
        msg = "compressed data must be bytes"
        raise TypeError(msg)
    return zlib.decompress(data)



from http import HTTPStatus
from typing import TYPE_CHECKING, Any, Literal
from typing_extensions import override

from nonebot.adapters import Bot as BaseBot

from nonebot.drivers import Request
from nonebot.message import handle_event
from yarl import URL

from .api import (
    UNSET,
    AllowedMention,
    ApiClient,
    File,
    InteractionCallbackMessage,
    InteractionCallbackType,
    InteractionResponse,
    MessageGet,
    MessageReference,
    Snowflake,
    SnowflakeType,
    User,
)
from .config import BotInfo
from .event import Event, InteractionCreateEvent, MessageEvent
from .exception import ActionFailed
from .message import Message, MessageSegment, parse_message
from .utils import log

if TYPE_CHECKING:
    from .adapter import Adapter


DISCORD_ATTACHMENT_HOSTS = {"cdn.discordapp.com", "media.discordapp.net"}
AttachmentFetchOnError = Literal["raise", "skip"]


async def _check_reply(bot: "Bot", event: MessageEvent) -> None:
    message_reference = event.message_reference
    if message_reference is UNSET:
        return

    message_id = message_reference.message_id
    if message_id is UNSET:
        return

    try:
        msg = await bot.get_channel_message(
            channel_id=event.channel_id, message_id=message_id
        )
        event.reply = msg
        if msg.author.id == bot.self_info.id:
            event.to_me = True
    except Exception as e:
        log("WARNING", f"Error when getting message reply info: {e!r}", e)


def _check_at_me(bot: "Bot", event: MessageEvent) -> None:  # noqa: C901
    if event.mentions is not None and bot.self_info.id in [
        user.id for user in event.mentions
    ]:
        event.to_me = True

    def _is_at_me_seg(segment: MessageSegment) -> bool:
        return (
            segment.type == "mention_user"
            and segment.data.get("user_id") == bot.self_info.id
        )

    message = event.get_message()

    # ensure message is not empty
    if not message:
        message.append(MessageSegment.text(""))

    deleted = False
    if _is_at_me_seg(message[0]):
        message.pop(0)
        deleted = True
        if message and message[0].type == "text":
            message[0].data["text"] = message[0].data["text"].lstrip("\xa0").lstrip()
            if not message[0].data["text"]:
                del message[0]

    if not deleted:
        # check the last segment
        i = -1
        last_msg_seg = message[i]
        if (
            last_msg_seg.type == "text"
            and not last_msg_seg.data["text"].strip()
            and len(message) >= 2  # noqa: PLR2004
        ):
            i -= 1
            last_msg_seg = message[i]

        if _is_at_me_seg(last_msg_seg):
            deleted = True
            del message[i:]

    if not message:
        message.append(MessageSegment.text(""))


class Bot(BaseBot, ApiClient):
    """
    Discord 协议 Bot 适配。
    """

    adapter: "Adapter"

    @override
    def __init__(self, adapter: "Adapter", self_id: str, bot_info: BotInfo) -> None:
        super().__init__(adapter, self_id)
        self.adapter = adapter
        self._bot_info: BotInfo = bot_info
        self._application_id: Snowflake = Snowflake(self_id)
        self._session_id: str | None = None
        self._self_info: User | None = None
        self._sequence: int | None = None

    @override
    def __repr__(self) -> str:
        return f"Bot(type={self.type!r}, self_id={self.self_id!r})"

    @property
    def ready(self) -> bool:
        return self._session_id is not None

    @property
    def bot_info(self) -> BotInfo:
        return self._bot_info

    @property
    def application_id(self) -> Snowflake:
        return self._application_id

    @property
    def session_id(self) -> str:
        if self._session_id is None:
            msg = f"Bot {self.self_id} is not connected!"
            raise RuntimeError(msg)
        return self._session_id

    @session_id.setter
    def session_id(self, session_id: str) -> None:
        self._session_id = session_id

    @property
    def self_info(self) -> User:
        if self._self_info is None:
            msg = f"Bot {self.bot_info} is not connected!"
            raise RuntimeError(msg)
        return self._self_info

    @self_info.setter
    def self_info(self, self_info: User) -> None:
        self._self_info = self_info

    @property
    def has_sequence(self) -> bool:
        return self._sequence is not None

    @property
    def sequence(self) -> int:
        if self._sequence is None:
            msg = f"Bot {self.self_id} is not connected!"
            raise RuntimeError(msg)
        return self._sequence

    @sequence.setter
    def sequence(self, sequence: int) -> None:
        self._sequence = sequence

    def clear(self) -> None:
        self._session_id = None
        self._sequence = None

    async def handle_event(self, event: Event) -> None:
        if isinstance(event, MessageEvent):
            await _check_reply(self, event)
            _check_at_me(self, event)
        await handle_event(self, event)

    async def fetch_attachments(  # noqa: PLR0913
        self,
        message: str | Message | MessageSegment,
        *,
        allowed_hosts: set[str] | None = None,
        require_https: bool = True,
        timeout: float | None = None,
        max_bytes: int | None = None,
        prefer_proxy_url: bool = True,
        on_error: AttachmentFetchOnError = "raise",
    ) -> Message:
        message = MessageSegment.text(message) if isinstance(message, str) else message
        message = message if isinstance(message, Message) else Message(message)
        new = message.clone()

        if allowed_hosts is None:
            allowed_hosts = DISCORD_ATTACHMENT_HOSTS

        attachment_segments = new["attachment"] or []
        for index, attachment in enumerate(attachment_segments):
            if attachment.data["file"] is not None:
                continue

            url = self._pick_attachment_url(
                attachment,
                allowed_hosts=allowed_hosts,
                require_https=require_https,
                prefer_proxy_url=prefer_proxy_url,
            )
            if url is None:
                if on_error == "raise":
                    msg = (
                        f"Attachment segment at index {index} has no fetchable "
                        "url/proxy_url"
                    )
                    raise ValueError(msg)
                continue

            content = await self._fetch_attachment_content(
                url,
                timeout=timeout,
                max_bytes=max_bytes,
            )
            if content is None:
                if on_error == "raise":
                    msg = (
                        f"Failed to fetch attachment content for segment "
                        f"at index {index} from URL {url}"
                    )
                    raise ValueError(msg)
                continue

            attachment.data["file"] = File(
                filename=attachment.data["attachment"].filename,
                content=content,
            )

        return new

    @staticmethod
    def _pick_attachment_url(
        attachment: MessageSegment,
        *,
        allowed_hosts: set[str],
        require_https: bool,
        prefer_proxy_url: bool,
    ) -> str | None:
        urls = []
        if prefer_proxy_url:
            urls.extend(
                [
                    attachment.data.get("proxy_url"),
                    attachment.data.get("url"),
                ]
            )
        else:
            urls.extend(
                [
                    attachment.data.get("url"),
                    attachment.data.get("proxy_url"),
                ]
            )

        for candidate in urls:
            if isinstance(candidate, str) and Bot._is_supported_attachment_url(
                candidate,
                allowed_hosts=allowed_hosts,
                require_https=require_https,
            ):
                return candidate

        return None

    @staticmethod
    def _is_supported_attachment_url(
        url: str, *, allowed_hosts: set[str], require_https: bool
    ) -> bool:
        parsed = URL(url)
        scheme_ok = parsed.scheme == "https" if require_https else bool(parsed.scheme)
        return (
            scheme_ok and isinstance(parsed.host, str) and parsed.host in allowed_hosts
        )

    async def _fetch_attachment_content(
        self,
        url: str,
        *,
        timeout: float | None,
        max_bytes: int | None,
    ) -> bytes | None:
        try:
            request = Request(
                method="GET",
                url=url,
                timeout=timeout or self.adapter.discord_config.discord_api_timeout,
                proxy=self.adapter.discord_config.discord_proxy,
            )
            response = await self.adapter.request(request)
            if response.status_code != HTTPStatus.OK or not response.content:
                return None
            content = (
                response.content.encode()
                if isinstance(response.content, str)
                else response.content
            )
            if max_bytes is not None and len(content) > max_bytes:
                return None
            return content  # noqa: TRY300
        except Exception as e:
            log("DEBUG", f"Failed to fetch attachment content from URL {url}: {e!r}", e)
            return None

    async def send_to(
        self,
        channel_id: SnowflakeType,
        message: str | Message | MessageSegment,
        tts: bool = False,  # noqa: FBT001, FBT002
        nonce: int | str | None = None,
        allowed_mentions: AllowedMention | None = None,
    ) -> MessageGet:
        message = MessageSegment.text(message) if isinstance(message, str) else message
        message = message if isinstance(message, Message) else Message(message)
        message = message.sendable()
        message_data = parse_message(message)

        return await self.create_message(
            channel_id=channel_id,
            nonce=nonce,
            tts=tts,
            allowed_mentions=allowed_mentions,
            **message_data,
        )

    @override
    async def send(
        self,
        event: Event,
        message: str | Message | MessageSegment,
        tts: bool = False,
        nonce: int | str | None = None,
        allowed_mentions: AllowedMention | None = None,
        mention_sender: bool | None = None,
        at_sender: bool | None = None,
        reply_message: bool = False,
        **params: Any,
    ) -> MessageGet:
        """send message.

        Args:
            event: Event Object
            message: message to send
            mention_sender: whether @ event subject
            reply_message: whether reply event message
            tts: whether send as a TTS message
            nonce: can be used to verify a message was sent
            allowed_mentions: allowed mentions for the message
            **params: other params

        Returns:
            message model
        """
        message = MessageSegment.text(message) if isinstance(message, str) else message
        message = message if isinstance(message, Message) else Message(message)
        message = message.sendable()
        if isinstance(event, InteractionCreateEvent):
            message_data = parse_message(message)
            response = InteractionResponse(
                type=InteractionCallbackType.CHANNEL_MESSAGE_WITH_SOURCE,
                data=InteractionCallbackMessage(
                    tts=tts, allowed_mentions=allowed_mentions, **message_data
                ),
            )
            try:
                await self.create_interaction_response(
                    interaction_id=event.id,
                    interaction_token=event.token,
                    response=response,
                )
            except ActionFailed:
                return await self.create_followup_message(
                    application_id=event.application_id,
                    interaction_token=event.token,
                    **message_data,
                )
            return await self.get_origin_interaction_response(
                application_id=event.application_id,
                interaction_token=event.token,
            )

        if not isinstance(event, MessageEvent) or not event.channel_id or not event.id:
            msg = "Event cannot be replied to!"
            raise RuntimeError(msg)
        if mention_sender or at_sender:
            message.insert(0, MessageSegment.mention_user(event.user_id))
        if reply_message:
            message += MessageSegment.reference(MessageReference(message_id=event.id))

        message_data = parse_message(message)

        return await self.create_message(
            channel_id=event.channel_id,
            nonce=nonce,
            tts=tts,
            allowed_mentions=allowed_mentions,
            **message_data,
        )
